@@ -9,7 +9,6 @@ import uuid
 import re
 
 from qtpy import QtCore, QtGui, QtWidgets
-from ipython_genutils.importstring import import_item
 
 from qtconsole.base_frontend_mixin import BaseFrontendMixin
 from traitlets import Any, Bool, Instance, Unicode, DottedObjectName, default
@@ -17,6 +16,7 @@ from .bracket_matcher import BracketMatcher
 from .call_tip_widget import CallTipWidget
 from .history_console_widget import HistoryConsoleWidget
 from .pygments_highlighter import PygmentsHighlighter
+from .util import import_item
 
 
 class FrontendHighlighter(PygmentsHighlighter):
@@ -189,9 +189,8 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
 
         # Configure actions.
         action = self._copy_raw_action
-        key = QtCore.Qt.CTRL | QtCore.Qt.SHIFT | QtCore.Qt.Key_C
         action.setEnabled(False)
-        action.setShortcut(QtGui.QKeySequence(key))
+        action.setShortcut(QtGui.QKeySequence("Ctrl+Shift+C"))
         action.setShortcutContext(QtCore.Qt.WidgetWithChildrenShortcut)
         action.triggered.connect(self.copy_raw)
         self.copy_available.connect(action.setEnabled)
@@ -465,7 +464,8 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         if info.kind == 'user':
             # Make sure that all output from the SUB channel has been processed
             # before writing a new prompt.
-            self.kernel_client.iopub_channel.flush()
+            if not self.kernel_client.iopub_channel.closed():
+                self.kernel_client.iopub_channel.flush()
 
             # Reset the ANSI style information to prevent bad text in stdout
             # from messing up our colors. We're not a true terminal so we're
@@ -505,7 +505,8 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
 
         # Make sure that all output from the SUB channel has been processed
         # before entering readline mode.
-        self.kernel_client.iopub_channel.flush()
+        if not self.kernel_client.iopub_channel.closed():
+            self.kernel_client.iopub_channel.flush()
 
         def callback(line):
             self._finalize_input_request()
@@ -538,6 +539,12 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         """
         self.log.warning("kernel restarted")
         self._kernel_restarted_message(died=died)
+
+        # This resets the autorestart counter so that the kernel can be
+        # auto-restarted before the next time it's polled to see if it's alive.
+        if self.kernel_manager:
+            self.kernel_manager.reset_autorestart_count()
+
         self.reset()
 
     def _handle_inspect_reply(self, rep):
@@ -733,7 +740,8 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         """Clears the current line of output."""
         cursor = self._control.textCursor()
         cursor.beginEditBlock()
-        cursor.movePosition(cursor.StartOfLine, cursor.KeepAnchor)
+        cursor.movePosition(QtGui.QTextCursor.StartOfLine,
+                            QtGui.QTextCursor.KeepAnchor)
         cursor.insertText('')
         cursor.endEditBlock()
 
@@ -830,8 +838,6 @@ class FrontendWidget(HistoryConsoleWidget, BaseFrontendMixin):
         """
         # Calculate where the cursor should be *after* the change:
         position += added
-
-        document = self._control.document()
         if position == self._get_cursor().position():
             self._auto_call_tip()
 
